@@ -408,46 +408,35 @@ class S7CPU(object):
 				   insn = oper.insn)
 
 	def __resolveSymbols_block(self, block):
-		#FIXME better error handling. Assign AwlSimError.insn
 		for insn in block.insns:
-			for i in range(len(insn.ops)):
-				insn.ops[i] = self.__resolveClassicSym(block,
-								insn, insn.ops[i])
-				insn.ops[i] = self.resolveNamedLocal(block,
-								insn, insn.ops[i],
-								pointer=False)
-				insn.ops[i] = self.__resolveNamedFullyQualified(block,
-								insn, insn.ops[i])
-				if insn.ops[i].type == AwlOperator.INDIRECT:
-					insn.ops[i].offsetOper = \
-						self.__resolveClassicSym(block,
-								insn, insn.ops[i].offsetOper)
-					insn.ops[i].offsetOper = \
-						self.resolveNamedLocal(block,
-								insn, insn.ops[i].offsetOper,
-								pointer=False)
-				insn.ops[i] = self.resolveNamedLocal(block,
-								insn, insn.ops[i],
-								pointer=True)
-			for i in range(len(insn.params)):
-				insn.params[i].rvalueOp = self.__resolveClassicSym(block,
-								insn, insn.params[i].rvalueOp)
-				insn.params[i].rvalueOp = self.resolveNamedLocal(block,
-								insn, insn.params[i].rvalueOp,
-								pointer=False)
-				insn.params[i].rvalueOp = self.__resolveNamedFullyQualified(block,
-								insn, insn.params[i].rvalueOp)
-				if insn.params[i].rvalueOp.type == AwlOperator.INDIRECT:
-					insn.params[i].rvalueOp.offsetOper =\
-						self.__resolveClassicSym(block,
-								insn, insn.params[i].rvalueOp.offsetOper)
-					insn.params[i].rvalueOp.offsetOper =\
-						self.resolveNamedLocal(block,
-								insn, insn.params[i].rvalueOp.offsetOper,
-								pointer=False)
-				insn.params[i].rvalueOp = self.resolveNamedLocal(block,
-								insn, insn.params[i].rvalueOp,
-								pointer=True)
+			try:
+				for i, oper in enumerate(insn.ops):
+					oper = self.__resolveClassicSym(block, insn, oper)
+					oper = self.resolveNamedLocal(block, insn, oper, False)
+					oper = self.__resolveNamedFullyQualified(block, insn, oper)
+					if oper.type == AwlOperator.INDIRECT:
+						oper.offsetOper = self.__resolveClassicSym(block,
+									insn, oper.offsetOper)
+						oper.offsetOper = self.resolveNamedLocal(block,
+									insn, oper.offsetOper, False)
+					oper = self.resolveNamedLocal(block, insn, oper, True)
+					insn.ops[i] = oper
+				for param in insn.params:
+					oper = param.rvalueOp
+					oper = self.__resolveClassicSym(block, insn, oper)
+					oper = self.resolveNamedLocal(block, insn, oper, False)
+					oper = self.__resolveNamedFullyQualified(block, insn, oper)
+					if oper.type == AwlOperator.INDIRECT:
+						oper.offsetOper = self.__resolveClassicSym(block,
+									insn, oper.offsetOper)
+						oper.offsetOper = self.resolveNamedLocal(block,
+									insn, oper.offsetOper, False)
+					oper = self.resolveNamedLocal(block, insn, oper, False)
+					param.rvalueOp = oper
+			except AwlSimError as e:
+				if not e.getInsn():
+					e.setInsn(insn)
+				raise e
 
 	# Resolve all symbols (global and local) on all blocks, as far as possible.
 	def __resolveSymbols(self):
@@ -823,12 +812,10 @@ class S7CPU(object):
 			return None
 
 	def labelIdxToRelJump(self, labelIndex):
+		# Translate a label index into a relative IP offset.
 		cse = self.callStackTop
-		label = cse.labels[labelIndex]
-		referencedInsn = label.getInsn()
-		referencedIp = referencedInsn.getIP()
-		assert(referencedIp < len(cse.insns))
-		return referencedIp - cse.ip
+		label = cse.block.labels[labelIndex]
+		return label.getInsn().getIP() - cse.ip
 
 	def jumpToLabel(self, labelIndex):
 		self.relativeJump = self.labelIdxToRelJump(labelIndex)
@@ -1436,6 +1423,8 @@ class S7CPU(object):
 	}
 
 	def __dumpMem(self, prefix, memArray, maxLen):
+		if not memArray:
+			return prefix + "--"
 		ret, line, first, count, i = [], [], True, 0, 0
 		while i < maxLen:
 			line.append("%02X" % memArray[i])
@@ -1453,11 +1442,13 @@ class S7CPU(object):
 	def __repr__(self):
 		if not self.callStack:
 			return ""
+		mnemonics = self.specs.getMnemonics()
+		isEnglish = (mnemonics == S7CPUSpecs.MNEMONICS_EN)
 		self.updateTimestamp()
 		ret = []
 		ret.append("=== S7-CPU dump ===  (t: %.01fs)" %\
 			   (self.now - self.startupTime))
-		ret.append("    STW:  " + str(self.statusWord))
+		ret.append("    STW:  " + self.statusWord.getString(mnemonics))
 		if self.is4accu:
 			accus = [ accu.toHex()
 				  for accu in (self.accu1, self.accu2,
@@ -1472,10 +1463,12 @@ class S7CPU(object):
 		ret.append(self.__dumpMem("      M:  ",
 					  self.flags,
 					  min(64, self.specs.nrFlags)))
-		ret.append(self.__dumpMem("    PAE:  ",
+		prefix = "      I:  " if isEnglish else "      E:  "
+		ret.append(self.__dumpMem(prefix,
 					  self.inputs,
 					  min(64, self.specs.nrInputs)))
-		ret.append(self.__dumpMem("    PAA:  ",
+		prefix = "      Q:  " if isEnglish else "      A:  "
+		ret.append(self.__dumpMem(prefix,
 					  self.outputs,
 					  min(64, self.specs.nrOutputs)))
 		pstack = str(self.callStackTop.parenStack) if self.callStackTop.parenStack else "Empty"
@@ -1487,9 +1480,16 @@ class S7CPU(object):
 			elems = " => ".join(elems)
 			ret.append("  Calls:  depth:%d   %s" %\
 				   (len(self.callStack), elems))
-			cse = self.callStack[-1]
+			localdata = self.callStack[-1].localdata
 			ret.append(self.__dumpMem("      L:  ",
-						  cse.localdata,
+						  localdata,
+						  min(16, self.specs.nrLocalbytes)))
+			try:
+				localdata = self.callStack[-2].localdata
+			except IndexError:
+				localdata = None
+			ret.append(self.__dumpMem("     VL:  ",
+						  localdata,
 						  min(16, self.specs.nrLocalbytes)))
 		else:
 			ret.append("  Calls:  None")
